@@ -12,14 +12,14 @@
 //! # #[cfg(feature = "yaxpeax")]
 //! # {
 //! # use yaxpeax_superh::{SuperHInstructionDecodeError, SuperHFloatRegister, SuperHInstruction, SuperHDecoder};
-//! # use yaxpeax_arch::Decoder;
-//! assert_eq!(SuperHDecoder::SH4.decode([0b0101_0101, 0b1100_1010].iter().copied()),
+//! # use yaxpeax_arch::{Decoder, U8Reader};
+//! assert_eq!(SuperHDecoder::SH4.decode(&mut U8Reader::new(&[0b0101_0101, 0b1100_1010])),
 //!            Ok(SuperHInstruction::XorImm(0x55)));
-//! assert_eq!(SuperHDecoder::SH4.decode([0b0101_1100, 0b1111_1010].iter().copied()),
+//! assert_eq!(SuperHDecoder::SH4.decode(&mut U8Reader::new(&[0b0101_1100, 0b1111_1010])),
 //!            Ok(SuperHInstruction::FMovS(SuperHFloatRegister::Fr5, SuperHFloatRegister::Fr10)));
-//! assert_eq!(SuperHDecoder::SH4.decode([0b0011_1000, 0b0000_0000].iter().copied()),
+//! assert_eq!(SuperHDecoder::SH4.decode(&mut U8Reader::new(&[0b0011_1000, 0b0000_0000])),
 //!            Ok(SuperHInstruction::Ldtlb));
-//! assert_eq!(SuperHDecoder::SH4.decode([0b0000_0001, 0b0000_0000].iter().copied()),
+//! assert_eq!(SuperHDecoder::SH4.decode(&mut U8Reader::new(&[0b0000_0001, 0b0000_0000])),
 //!            Err(SuperHInstructionDecodeError::NotFound));
 //! # }
 //! # }
@@ -36,7 +36,7 @@
 //! # {
 //! # use self::yaxpeax_superh::{SuperHInstructionDecodeError, SuperHExtendedDoubleRegister, SuperHDoubleRegister,
 //! #                            SuperHInstruction, SuperHFeature, SuperHDecoder};
-//! # use self::yaxpeax_arch::Decoder;
+//! # use self::yaxpeax_arch::{Decoder, U8Reader};
 //! let cpu = SuperHDecoder {
 //!     little_endian: false,
 //!     fpscr_sz: true,
@@ -44,14 +44,14 @@
 //!     ..SuperHDecoder::SH4
 //! };
 //!
-//! assert_eq!(cpu.decode([0b1100_1010, 0b0101_0101].iter().copied()),
+//! assert_eq!(cpu.decode(&mut U8Reader::new(&[0b1100_1010, 0b0101_0101])),
 //!            Ok(SuperHInstruction::XorImm(0x55)));
-//! assert_eq!(cpu.decode([0b1111_1010, 0b0101_1100].iter().copied()),
+//! assert_eq!(cpu.decode(&mut U8Reader::new(&[0b1111_1010, 0b0101_1100])),
 //!            Ok(SuperHInstruction::FMovFromX(SuperHExtendedDoubleRegister::Xd4,
 //!                                            SuperHDoubleRegister::Dr10)));
-//! assert_eq!(cpu.decode([0b0000_0000, 0b0011_1000].iter().copied()),
+//! assert_eq!(cpu.decode(&mut U8Reader::new(&[0b0000_0000, 0b0011_1000])),
 //!            Err(SuperHInstructionDecodeError::NotFound));
-//! assert_eq!(cpu.decode([0b0000_0000, 0b0000_0001].iter().copied()),
+//! assert_eq!(cpu.decode(&mut U8Reader::new(&[0b0000_0000, 0b0000_0001])),
 //!            Err(SuperHInstructionDecodeError::NotFound));
 //! # }
 //! # }
@@ -106,13 +106,15 @@
 
 #[cfg(feature = "yaxpeax")]
 extern crate yaxpeax_arch;
+#[cfg(feature = "std")]
+extern crate std;
 
 use core::fmt;
 use core::convert::TryFrom;
 #[cfg(feature = "yaxpeax")]
 use core::marker::PhantomData;
 #[cfg(feature = "yaxpeax")]
-use yaxpeax_arch::AddressDiff;
+use yaxpeax_arch::{AddressDiff, Arch, DecodeError};
 
 
 #[derive(Debug, Copy, Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
@@ -1081,7 +1083,7 @@ pub enum SuperHInstruction {
 
     /// CAS.L Rm,Rn,@R0 (When a byte in Rn equals a byte in Rm, 1 → `T`) `0010nnnnmmmm0011`
     ///
-    /// https://github.com/j-core/jcore-cpu/blob/00f398e/decode/decode_table_simple.vhd#L1919
+    /// <https://github.com/j-core/jcore-cpu/blob/00f398e/decode/decode_table_simple.vhd#L1919>
     ///
     /// Features: J2
     /// Level: J2
@@ -1809,15 +1811,15 @@ pub enum SuperHInstructionDecodeError {
 #[cfg(feature = "yaxpeax")]
 impl fmt::Display for SuperHInstructionDecodeError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            SuperHInstructionDecodeError::Exhausted => f.write_str("exhausted input"),
-            SuperHInstructionDecodeError::NotFound => f.write_str("invalid opcode"),
-        }
+        f.write_str(self.description())
     }
 }
 
+#[cfg(all(feature = "yaxpeax", feature = "std"))]
+impl std::error::Error for SuperHInstructionDecodeError {}
+
 #[cfg(feature = "yaxpeax")]
-impl yaxpeax_arch::DecodeError for SuperHInstructionDecodeError {
+impl DecodeError for SuperHInstructionDecodeError {
     fn data_exhausted(&self) -> bool {
         *self == SuperHInstructionDecodeError::Exhausted
     }
@@ -1829,6 +1831,20 @@ impl yaxpeax_arch::DecodeError for SuperHInstructionDecodeError {
     fn bad_operand(&self) -> bool {
         false
     }
+
+    fn description(&self) -> &'static str {
+        match self {
+            SuperHInstructionDecodeError::Exhausted => "exhausted input",
+            SuperHInstructionDecodeError::NotFound => "invalid opcode",
+        }
+    }
+}
+
+#[cfg(feature = "yaxpeax")]
+impl From<yaxpeax_arch::ReadError> for SuperHInstructionDecodeError {
+    fn from(_e: yaxpeax_arch::ReadError) -> SuperHInstructionDecodeError {
+        SuperHInstructionDecodeError::Exhausted
+    }
 }
 
 #[cfg(feature = "yaxpeax")]
@@ -1839,7 +1855,7 @@ pub struct SuperHDecoder<'f> {
     /// When `FPSCR.SZ` = 1, the SH-4 CPU core can perform data transfer by means of pair single-precision data transfer
     /// instructions.
     ///
-    /// http://www.st.com/st-web-ui/static/active/en/resource/technical/document/user_manual/CD00147165.pdf
+    /// <http://www.st.com/st-web-ui/static/active/en/resource/technical/document/user_manual/CD00147165.pdf>
     /// Section 6.5.2 "Pair single-precision data transfer" p. 154
     pub fpscr_sz: bool,
 
@@ -1891,19 +1907,18 @@ impl SuperHDecoder<'_> {
 }
 
 #[cfg(feature = "yaxpeax")]
-impl yaxpeax_arch::Decoder<SuperHInstruction> for SuperHDecoder<'_> {
-    type Error = SuperHInstructionDecodeError;
-
-    fn decode_into<T: IntoIterator<Item = u8>>(&self, inst: &mut SuperHInstruction, bytes: T) -> Result<(), Self::Error> {
-        *inst = self.decode(bytes)?;
+impl<'f> yaxpeax_arch::Decoder<SuperH<'_>> for SuperHDecoder<'_> {
+    fn decode_into<T: yaxpeax_arch::Reader<<SuperH<'f> as Arch>::Address, <SuperH<'f> as Arch>::Word>>(&self, inst: &mut SuperHInstruction, words: &mut T)
+                                                                                                       -> Result<(), <SuperH<'_> as Arch>::DecodeError> {
+        *inst = self.decode(words)?;
         Ok(())
     }
 
-    fn decode<T: IntoIterator<Item = u8>>(&self, bytes: T) -> Result<SuperHInstruction, Self::Error> {
+    fn decode<T: yaxpeax_arch::Reader<<SuperH<'f> as Arch>::Address, <SuperH<'f> as Arch>::Word>>(
+        &self, words: &mut T)
+        -> Result<SuperHInstruction, <SuperH<'_> as Arch>::DecodeError> {
         // mov #0x69,r4 (1110nnnniiiiiiii) encodes as 69 e4 in LE and e4 69 in BE
-        let mut bytes = bytes.into_iter();
-        let ins = ((bytes.next().ok_or(SuperHInstructionDecodeError::Exhausted)? as u16) << (if self.little_endian { 0 } else { 8 })) |
-                  ((bytes.next().ok_or(SuperHInstructionDecodeError::Exhausted)? as u16) << (if self.little_endian { 8 } else { 0 }));
+        let ins = ((words.next()? as u16) << (if self.little_endian { 0 } else { 8 })) | ((words.next()? as u16) << (if self.little_endian { 8 } else { 0 }));
 
         let ret = SuperHInstruction::parse(ins, self.fpscr_sz).ok_or(SuperHInstructionDecodeError::NotFound)?;
 
@@ -1935,8 +1950,9 @@ impl<'f> Default for SuperHDecoder<'f> {
 #[cfg(feature = "yaxpeax")]
 pub struct SuperH<'f>(PhantomData<&'f ()>);
 #[cfg(feature = "yaxpeax")]
-impl<'f> yaxpeax_arch::Arch for SuperH<'f> {
+impl<'f> Arch for SuperH<'f> {
     type Address = u32;
+    type Word = u8;
     type Instruction = SuperHInstruction;
     type DecodeError = SuperHInstructionDecodeError;
     type Decoder = SuperHDecoder<'f>;
